@@ -2,14 +2,15 @@
 # SPDX-License-Identifier: GPL-3.0-or-later
 import collections
 import glob
-from pmb.helpers import logging
 import os
 from pathlib import Path
-from typing import Any, TypedDict
+from typing import TypedDict
+
 import pmb.chroot
-from pmb.types import Env, PmbArgs
 import pmb.helpers.cli
 from pmb.core import Chroot
+from pmb.helpers import logging
+from pmb.types import Env, RunOutputTypeDefault
 
 
 class CiScriptDescriptor(TypedDict):
@@ -19,7 +20,8 @@ class CiScriptDescriptor(TypedDict):
 
 
 def get_ci_scripts(topdir: Path) -> dict[str, CiScriptDescriptor]:
-    """Find 'pmbootstrap ci'-compatible scripts inside a git repository, and
+    """
+    Find 'pmbootstrap ci'-compatible scripts inside a git repository, and
     parse their metadata (description, options). The reference is at:
     https://postmarketos.org/pmb-ci
 
@@ -69,7 +71,8 @@ def get_ci_scripts(topdir: Path) -> dict[str, CiScriptDescriptor]:
 
 
 def sort_scripts_by_speed(scripts: dict[str, CiScriptDescriptor]) -> dict[str, CiScriptDescriptor]:
-    """Order the scripts, so fast scripts run before slow scripts. Whether a
+    """
+    Order the scripts, so fast scripts run before slow scripts. Whether a
     script is fast or not is determined by the '# Options: slow' comment in
     the file.
 
@@ -98,7 +101,8 @@ def sort_scripts_by_speed(scripts: dict[str, CiScriptDescriptor]) -> dict[str, C
 def ask_which_scripts_to_run(
     scripts_available: dict[str, CiScriptDescriptor],
 ) -> dict[str, CiScriptDescriptor]:
-    """Display an interactive prompt about which of the scripts the user
+    """
+    Display an interactive prompt about which of the scripts the user
     wishes to run, or all of them.
 
     :param scripts_available: same format as get_ci_scripts()
@@ -107,7 +111,7 @@ def ask_which_scripts_to_run(
 
     """
     count = len(scripts_available.items())
-    choices = ["all"]
+    choices = []
 
     logging.info(f"Available CI scripts ({count}):")
     for script_name, script in scripts_available.items():
@@ -117,26 +121,32 @@ def ask_which_scripts_to_run(
         logging.info(f"* {script_name}: {script['description']}{extra}")
         choices += [script_name]
 
-    selection = pmb.helpers.cli.ask("Which script?", None, "all", complete=choices)
+    choice_regex = "|".join(choices)
+    ci_regex = f"(all)|({choice_regex})(,({choice_regex}))*"
+    choices += ["all"]
+    logging.info('Fill a comma-separated list of scripts or "all"')
+    selection = pmb.helpers.cli.ask(
+        "Scripts", None, "all", validation_regex=ci_regex, complete=choices
+    )
     if selection == "all":
         return scripts_available
-
-    ret = {}
-    ret[selection] = scripts_available[selection]
-    return ret
+    else:
+        return {script: scripts_available[script] for script in selection.split(",")}
 
 
-def copy_git_repo_to_chroot(topdir: Path) -> None:
-    """Create a tarball of the git repo (including unstaged changes and new
+def copy_git_repo_to_chroot(topdir: Path, include_dot_git_dir: bool = False) -> None:
+    """
+    Create a tarball of the git repo (including unstaged changes and new
     files) and extract it in chroot_native.
 
     :param topdir: top directory of the git repository, get it with:
       pmb.helpers.git.get_topdir()
+    :param include_dot_git_dir: Also include .git
     """
     chroot = Chroot.native()
     pmb.chroot.init(chroot)
     tarball_path = chroot / "tmp/git.tar.gz"
-    files = pmb.helpers.git.get_files(topdir)
+    files = pmb.helpers.git.get_files(topdir, include_dot_git_dir=include_dot_git_dir)
 
     with open(f"{tarball_path}.files", "w") as handle:
         for file in files:
@@ -152,7 +162,8 @@ def copy_git_repo_to_chroot(topdir: Path) -> None:
 
 
 def run_scripts(topdir: Path, scripts: dict[str, CiScriptDescriptor]) -> None:
-    """Run one of the given scripts after another, either natively or in a
+    """
+    Run one of the given scripts after another, either natively or in a
     chroot. Display a progress message and stop on error (without printing
     a python stack trace).
 
@@ -177,17 +188,23 @@ def run_scripts(topdir: Path, scripts: dict[str, CiScriptDescriptor]) -> None:
         logging.info(f"*** ({step}/{steps}) RUNNING CI SCRIPT: {script_path} [{where}] ***")
 
         if "native" in script["options"]:
-            rc = pmb.helpers.run.user([script_path], topdir, output="tui")
+            rc = pmb.helpers.run.user([script_path], topdir, output=RunOutputTypeDefault.TUI)
             continue
         else:
             # Run inside pmbootstrap chroot
             if not repo_copied:
-                copy_git_repo_to_chroot(topdir)
+                copy_git_repo_to_chroot(
+                    topdir, include_dot_git_dir="with-dot-git" in script["options"]
+                )
                 repo_copied = True
 
             env: Env = {"TESTUSER": "pmos"}
             rc = pmb.chroot.root(
-                [script_path], check=False, env=env, working_dir=Path("/home/pmos/ci"), output="tui"
+                [script_path],
+                check=False,
+                env=env,
+                working_dir=Path("/home/pmos/ci"),
+                output=RunOutputTypeDefault.TUI,
             )
         if rc:
             logging.error(f"ERROR: CI script failed: {script_name}")
